<?php

/* This is Public Domain code written by Greg Jennings. MRU February 2014. */

/*
This is a "debug" API. It is very simple yet very versatile. Just include this 
file, enable debugging and optionally set some options, and then strategically 
place calls to the debug() function in and around the code you want to debug.
The debug() function takes anything as an argument which will be displayed 
at program termination in a list. This output can be customised.

See the file "debug.html" for more information.
*/

if (!defined('DEBUG_MSG')) define('DEBUG_MSG','<b>message log:</b>');
if (!defined('DEBUG_OPEN')) define('DEBUG_OPEN',"\n<div style='background:#fff; color:#000;'><pre>");
if (!defined('DEBUG_CLOSE')) define('DEBUG_CLOSE','</pre></div>');
if (!defined('DEBUG_EMPTY')) define('DEBUG_EMPTY',0);
if (!defined('DEBUG_CALLER')) define('DEBUG_CALLER',1);
if (!defined('DEBUG_NEWLINE')) define('DEBUG_NEWLINE',1);
if (!defined('DEBUG_ENTITIES')) define('DEBUG_ENTITIES',1);

$GLOBALS['debug_msgs'] = '';

/* debuglog - set the errorlog level */

function debuglog($log) {

	_debug('log',$log);
	if ($log && !_debug('reg')) {
		_debug('reg',1);
		register_shutdown_function('_msglogdump');
	}
}

/* debug - display message or add to message queue */

function debug($msg = NULL, $arg = 0) {

	$log = _debug('log');
	if ($msg === NULL)
		return $log;
	if ($log == 0)
		return;
	if ($arg == -1) {
		$arg = 0;
		$log = -1;
	}

	list($file,$line,$func) = _caller(1);
	$func = _caller(2,'function');

	if (($t = _debug('file')) && $t != $file)
		return;

	if ($arg === 'type') {
		$msg = gettype($msg);
		$arg = 0;
	}
	else
	if ($msg === TRUE)
		$msg = '(true)';
	else
	if ($msg === FALSE)
		$msg = '(false)';
	else
	if ($msg === '')
		$msg = '(\'\')';
	else
	if (is_resource($msg))
		$msg = 'Resource: '.get_resource_type($msg);
	else
	if (is_object($msg))
		$msg = 'Object: '.get_class($msg);
	else
	if (is_array($msg))
		$msg = 'Array: {'. _flatten($msg) .'}';

	if ($arg == 0 && ($t = _debug('trunc')))
		$arg = $t;

	if ($arg > 0 && strlen($msg) > $arg)
		$msg = substr($msg,0,$arg).' ...';

	if (_debug('entities'))
		$msg = htmlentities($msg);

	if (_debug('newline'))
		$msg = str_replace("\n",'\n',$msg);

	$c = '';
	if (_debug('caller')) {
		list($f,$l,$na) = _caller(2);
		$c = "($f,$l)";
	}

	if (_debug('nofunc'))
		$msg = "$c($file,$line) $msg";
	else
		$msg = "$c($file,$line,$func) $msg";

	if ($log == -1)
		print $msg . '<br>';
	else
	if ($log == 100)
		print '<!-- '.$msg.' -->'.PHP_EOL;
	else
	if ($log == 200)
		$GLOBALS['debug_msgs'] .= $msg.'<br>'.PHP_EOL;
	else
		_debug('msg',$msg);
}

/* _msglogdump - display the message log (as a shutdown fuction) */

function _msglogdump() {

	$log = _debug('log');

	if ($log <= 0 || $log >= 100)		// special cases
		return;

	$debug = _debug();
	if (count($debug['msgs']) == 0 && !$debug['empty'])
		return;

	print _debug('open');
	print _debug('msg');
	foreach (_debug('msgs') as $msg)
		print "\n$msg";
	print _debug('close');
}

/* _debug - debug handler initialize and configuration */

function _debug($key = NULL, $val = NULL) {
static $debug = array();

	if ($debug == array()) {
		$debug['log'] = 0;
		$debug['msgs'] = array();
		$debug['msg'] = DEBUG_MSG;
		$debug['open'] = DEBUG_OPEN;
		$debug['close'] = DEBUG_CLOSE;
		$debug['empty'] = DEBUG_EMPTY;
		$debug['caller'] = DEBUG_CALLER;
		$debug['newline'] = DEBUG_NEWLINE;
		$debug['entities'] = DEBUG_ENTITIES;

		if (defined('DEBUG_LOG'))		// do last
			debuglog(DEBUG_LOG);		//  recurses
	}

	if ($key === NULL)				// return array
		return $debug;

	if ($key == 'msg')				// add a message
		return $debug['msgs'][] = $val;

	if ($val !== NULL)				// set a value
		return $debug[$key] = $val;

	if (isset($debug[$key]))			// get a value
		return $debug[$key];

	return '';
}

/* _flatten - an implode() that handles associative arrays */

function _flatten($a) {

	if (isset($a[0]) && !is_array($a[0]))
		return implode(',',$a);

	$s = '';
	foreach ($a as $k => $v) {
		if (is_array($v))
			$s .= "$k="._flatten($v);
		else
			$s .= "$k='$v',";
	}

	return rtrim($s,',');
}

/* _caller - get caller of caller (why is this function so big) */

function _caller($depth = 1, $elem = NULL, $nb = 0, $rbt = 0) {

	if (defined('DEBUG_BACKTRACE_IGNORE_ARGS'))
		$bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
	else
		$bt = debug_backtrace();

	if ($rbt)
		return $bt;

	if ($elem) {
		$s = (@$bt[$depth][$elem]) ? $bt[$depth][$elem] : '';
		if (!$nb && $elem == 'file')
			$s = basename($s);
		return $s;
	}

	if (!isset($bt[$depth]))
		return(array('?',$depth,''));

	$bt = $bt[$depth];
	if ($nb) 
		$file = (@$bt['file']) ? $bt['file'] : '';
	else
		$file = (@$bt['file']) ? basename($bt['file']) : '';
	$line = (@$bt['line']) ? $bt['line'] : '';
	$func = $bt['function'];

	if ($elem === '')
		return "$file,$line,$func";

	return array($file,$line,$func);
}

/* error_handler - an example error handler */

function error_handler($errno, $errstr, $errfile, $errline) {

	$errfile = basename($errfile);
	$error = "$errstr ($errfile, $errline)";

	if ($errno == E_USER_ERROR)
        	exit($error);

	if ($errno == E_NOTICE) {
		if (_debug('shownotices'))
			echo($error.'<br>');
		if (_debug('debugnotices'))
			debug($error);
		return TRUE;
	}

	if (_debug('showerrors'))
		echo($error.'<br>');

	debug($error);

	return TRUE;
}
